/*
 * Scratchpad MMU - Hardware driver for test program
 * Copyright (C) Jack Whitham 2009
 * http://www.jwhitham.org.uk/c/smmu.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation
 * (version 2.1 of the License only).
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
#ifndef HWDRIVER_H
#define HWDRIVER_H

/* Control register locations */
typedef struct Control_Registers_struct {
        /* These are the read-only names of the registers.
         * When writing, some of them take on different functions. */
        unsigned        status ;            /* write: OPEN/CLOSE control */
        unsigned        global_counter ;    /* write: Offset register */
        unsigned        match_count ;       /* write: Comment register */
        unsigned        miss_count ;        /* write: Cache control reg */
        unsigned        dbg_code ;
        unsigned        reserved1 ;
        unsigned        configuration_2 ;
        unsigned        configuration ;
} Control_Registers ;

static volatile Control_Registers * const CONTROL = 
            (volatile Control_Registers *) 0xa0000000UL ;
static volatile unsigned * const SRAM =
            (volatile unsigned *) 0x90000000UL ;

#define CONTROL_OFFSET  CONTROL->global_counter
#define CONTROL_COMMENT CONTROL->match_count

/* Static setup */
#define MAX_NUM_ENTRIES     256

/* SMMU configuration: discovered by HW_Init() */
extern unsigned NUM_RO_ENTRIES ;
extern unsigned NUM_ENTRIES ;       /* includes zero entry */
extern unsigned NUM_RW_ENTRIES ;
extern unsigned FIRST_RW_ENTRY ;
extern unsigned HAS_CACHE ;
extern unsigned HAS_EXTRA_SP ;

extern unsigned LOG2_SP_SIZE ; /* words */
extern unsigned LOG2_NUM_CACHE_WORDS_PER_LINE ;
extern unsigned LOG2_NUM_CACHE_LINES_PER_PAGE ;
extern unsigned LOG2_NUM_CACHE_PAGES ;

extern unsigned SP_SIZE ; /* in words */
extern unsigned NUM_CACHE_WORDS_PER_LINE ;
extern unsigned NUM_CACHE_LINES_PER_PAGE ;
extern unsigned NUM_CACHE_PAGES ;

/* Functions that work with logical addresses */
unsigned HW_OPEN ( void * address , unsigned size , unsigned copy_loc ) ;
unsigned HW_OPEN_RO ( void * address , unsigned size , unsigned copy_loc ) ;

/* Functions that don't use indexes or logical addresses */
void HW_CLOSE ( unsigned ref ) ;
unsigned HW_Get_Result_Counter ( void ) ; 
unsigned HW_Get_Result_Timer ( void ) ;
void HW_Init ( void ) ;
int HW_Probe ( void ) ;

/* Cache functions */
void HW_Cache_Set_Permutation (unsigned src, unsigned dest);
void HW_Cache_Enable ( unsigned flush );
void HW_Cache_Disable ();

#endif

