
library ieee;
library unisim ;

use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned."+";
use ieee.std_logic_unsigned."-";
use ieee.std_logic_unsigned.conv_integer;
use unisim.vcomponents.all ;


entity comp_network is
    generic ( 
        ACTIVE_LEFT         : Natural;
        NUM_ENTRIES         : Natural );
    port (
        match_in_address    : in std_logic_vector(ACTIVE_LEFT downto 0);
        min_address_cat     : in std_logic_vector(
                    ((ACTIVE_LEFT + 1) * NUM_ENTRIES) - 1 downto 0);
        max_address_cat     : in std_logic_vector(
                    ((ACTIVE_LEFT + 1) * NUM_ENTRIES) - 1 downto 0);
        row_match_1         : out std_logic_vector(0 to NUM_ENTRIES-1);
        row_match_2         : out std_logic_vector(0 to NUM_ENTRIES-1));
end entity comp_network;

architecture basic of comp_network is
begin
    process ( match_in_address , min_address_cat, max_address_cat ) is
        variable x, y : Natural;
    begin
        row_match_1 <= ( others => '0' );
        row_match_2 <= ( others => '0' );
        for row in 1 to ( NUM_ENTRIES - 1 )
        loop
            -- Comparison and out_address computation
            -- are done in parallel
            y := row * (ACTIVE_LEFT + 1);
            x := y + ACTIVE_LEFT;
            if ( min_address_cat(x downto y) <= match_in_address )
            then
                row_match_1(row) <= '1';
            end if;
            if ( max_address_cat(x downto y) > match_in_address )
            then
                row_match_2(row) <= '1';
            end if;
        end loop;
    end process;
end architecture basic;

