# 
#  Scratchpad MMU - Test program module 3 - Code Generator
#  Copyright (C) Jack Whitham 2010
#  http://www.jwhitham.org.uk/c/smmu.html
# 
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation
#  (version 2.1 of the License only).
#  
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#  
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
# 

import random

TABLE_SIZE = 79 # Max 255
KEY = random.randrange(0, 1 << 16)


def Emit_Word(v):
    assert 0 <= v < (1 << 16)
    print '.byte 0x%02x' % (v >> 8)
    print '.byte 0x%02x' % (v & 0xff)

tableA = [ random.randrange(0, 1 << 16) for i in xrange(TABLE_SIZE) ]
tableB = [ random.randrange(0, 1 << 16) for i in xrange(TABLE_SIZE) ]
print """
.data
.align 2
tableA:
"""
for val in tableA:
    Emit_Word(val)

print 'tableB:'
for val in tableB:
    Emit_Word(val)

print 'commands:'
tableC = dict()
spaces = range(TABLE_SIZE)
random.shuffle(spaces)
sources = range(TABLE_SIZE)
sources += range(TABLE_SIZE)
random.shuffle(sources)

for i in xrange(TABLE_SIZE):
    srcA = sources.pop()
    srcB = sources.pop()
    dest = spaces.pop()
    tableC[ dest ] = tableA[ srcA ] ^ tableB[ srcB ] ^ KEY
    val = (srcA << 16) | (srcB << 8) | dest
    print '.word 0x%08x' % val

assert len(tableC) == TABLE_SIZE
assert len(sources) == 0
assert len(spaces) == 0

print '.align 4'
print 'tableC:'
for i in xrange(TABLE_SIZE):
    Emit_Word(tableC[ i ])

print """
.align 2
.text
.align 2
.globl braindamage_test
braindamage_test:
    addi    r1, r1, -0x100
"""
# Save all registers
for i in xrange(4, 32, 1):
    print 'swi     r%d, r1, 0x%x' % (i, i * 4)

print """
    addi    r31, r1, 0
    addi    r1, r1, %(TABLE_SIZE)u * -2
    ori     r1, r1, 0x3
    xori    r1, r1, 0x3

    /* Loop 1 - execute commands */
    addik   r3, r0, commands
    addik   r4, r0, %(TABLE_SIZE)u
0:
    lw      r5, r3, r0      /* Load command */
    bsrli   r6, r5, 16      /* Obtain srcA */
    bsrli   r7, r5, 8       /* Obtain srcB */
    andi    r7, r7, 0xff
    andi    r8, r5, 0xff    /* Obtain dest */
    bslli   r6, r6, 1       /* srcA converted to array index */
    bslli   r7, r7, 1       /* srcB likewise */
    bslli   r8, r8, 1       /* dest likewise */
    lhui    r6, r6, tableA  /* get input A */
    lhui    r7, r7, tableB  /* get input B */
    xor     r9, r6, r7      /* compute output */
    sh      r9, r1, r8      /* store output on stack */
    addik   r4, r4, -1      /* Loop iterator */
    bneid   r4, 0b          /* Next iteration */
    addik   r3, r3, 4       /* Command iterator */

    /* Loop 2 - check output in word mode */
    addk    r3, r0, r1
    addik   r5, r0, tableC
    addik   r4, r0, %(TABLE_SIZE)u / 2
0:
    lw      r6, r0, r3
    lw      r7, r0, r5
    xor     r8, r6, r7
    xori    r8, r8, 0x%(KEY)04x%(KEY)04x
    bneid   r8, explosion
    
    addik   r5, r5, 4       /* Word (tableC) */
    addik   r4, r4, -1      /* Loop iterator */
    bneid   r4, 0b          /* Next iteration */
    addik   r3, r3, 4       /* Word (stack) */

    /* Loop 3 - check output in byte mode */
    addk    r3, r0, r1
    addik   r5, r0, tableC
    addik   r4, r0, %(TABLE_SIZE)u 
0:
    lbu     r6, r0, r3
    lbu     r7, r0, r5
    xor     r8, r6, r7
    xori    r8, r8, %(KEY)u >> 8
    bneid   r8, explosion1

    lbui    r6, r3, 1
    lbui    r7, r5, 1
    xor     r8, r6, r7
    xori    r8, r8, %(KEY)u & 0xff
    bneid   r8, explosion1
    
    addik   r5, r5, 2       /* Byte (tableC) */
    addik   r4, r4, -1      /* Loop iterator */
    bneid   r4, 0b          /* Next iteration */
    addik   r3, r3, 2       /* Byte (stack) */

    /* All done, and everything worked */
    addik   r3, r0, 0       /* Return 0 */
clean_exit:
    addik   r1, r31, 0
""" % globals()

# Restore all registers
for i in xrange(4, 32, 1):
    print 'lwi     r%d, r1, 0x%x' % (i, i * 4)

print """
    rtsd    r15, 8
    addik   r1, r1, 0x100
explosion:
    xori    r3, r4, 0x400
    bri     clean_exit
explosion1:
    xori    r3, r4, 0x100
    bri     clean_exit

"""




